# mad/get.py
#
#

""" functions that can be used on objects. """

from bot.time import elapsed, to_date, fn_time, to_time
from bot.time import rtime as _rtime

import bot.error
import time
import bot
import os

dirmask = 0o700
filemask = 0o600

def mi(name):
    """ grab an object from the mid module. """
    import mid
    return getattr(mid, name, None)

def fnn(object):
    """ grep the file name from a code object. """
    fn = object.f_code.co_filename
    return fn.split(os.sep)[-1][:-3]

def urled(object):
    """ return a url for the object so it can be fetched with the REST service. """
    from bot.space import cfg
    p = get_path(object)
    p = p.split(cfg.workdir)[-1]
    return "http://%s:%s%s" % (cfg.hostname or "localhost", cfg.port, p)

def root():
    """ return the root directory. """
    from bot.space import cfg
    path = cfg.workdir
    path = os.path.abspath(path)
    bot.file.check_permissions(path)
    return path

def days(object):
    """ calculate the time passed since an object got logged. """
    t1 = time.time()
    t2 = timed(object)
    if t2:
        time_diff = float(t1 - t2)
        return elapsed(time_diff)

def dated(object):
    """ fetch the date from an object. """
    res = ""
    if "_container" in object:
        object = object._container
    if not res:
        res = getattr(object, "Date", None)
    if not res:
        res = getattr(object, "date", None)
    if not res:
        res = getattr(object, "published", None)
    if not res:
        res = getattr(object, "added", None)
    if not res:
        res = getattr(object, "saved", None)
    if not res:
        res = getattr(object, "timed", None)
    if not res:
        raise bot.error.ENODATE
    return res

def timed(object):
    """ calculated the time of an object. """
    try:
        date = dated(object._container)
    except (AttributeError, bot.error.ENODATE):
        try:
           date = dated(object)
        except bot.error.ENODATE:
           date = None
    if date:
        try: 
            return to_time(date)
        except bot.error.ENODATE:
            pass
    try:
        return fn_time(object._container.path)
    except:
        try:
            return fn_time(rtime())
        except:
           pass

def slice(object, keys=[], skip=[], full=False):
    """ return a slice of an object. """
    from bot.object import Object
    o = Object()
    if not keys:
        keys = object.keys()
    for key in keys:
        if key in skip:
            continue
        if not full and key.startswith("_"):
            continue
        val = object.get(key, None)
        if val and "keys" in dir(val):
            o[key] = slice(val)
        else:
            o[key] = val
    return o

def get_path(object):
    """ Return the path used to store the object's json dump. """
    try:
        return object._container.path
    except:
        pass

def get_saved(object):
    """ return the saved attribue of an object. """
    p = ""
    if "_container" in object:
        p = getattr(object._container, "saved", "")
        if p:
            return p
