# mad/kernel.py
#
#

""" module loading. """

from bot.file import locatedir
from bot.join import j
from bot.log import loglevel
from bot.misc import locked
from bot.name import pathname
from bot.shell import set_completer, parse_cli, banner
from bot.trace import get_exception
from bot.template import template

from bot.object import Default, Object, Config
from bot.register import Register
from bot.db import Db
from bot.handler import Handler
from bot.event import Event

from bot import __license__

import bot
import logging
import importlib
import os
import pkgutil
import queue
import time
import types

class Kernel(Handler, Db):


    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._booted = Object()
        self._cbs = Register()
        self._cmnds = []
        self._finished = Object()
        self._names = Register()
        self._table = Object()
        self._time = Default(default=0)
     
    def announce(self, txt):
        """ announce txt on all fleet bot. """
        from bot.space import fleet
        for bot in fleet.bots:
            bot.announce(txt)

    def boot(self, name="MADBOT", version=bot.__version__, txt=bot.__txt__, **kwargs):
        """ start the kernel. """
        from bot.space import alias, cfg, fleet, load, users, ENOWORKDIR
        self._time.start = time.time()
        main = parse_cli()
        main.update(kwargs)
        if main.yes or main.banner:
            main.shell = True
            main.test = True
            main.loglevel = main.loglevel or "warn"
            main.workdir = "data"
        if main.yes or main.shell:
            main.loglevel = main.loglevel or "error"
            main.init += ",cli"
        loglevel(main.loglevel or "error", main.colors)
        if main.yes or main.banner:
            banner(name, version, txt, **kwargs)
        if main.license:
            print(__license__)
        if not main.workdir:
            if main.local:
                main.workdir = kwargs.get("workdir", j(main.homedir, ".madbot"))
            else:
                main.workdir = kwargs.get("workdir", j(os.getcwd(), "data"))
        else:
            main.changed = True
        if not os.path.isdir(main.workdir):
            os.mkdir(main.workdir)
        cfg.update(main)
        thrs = []
        dosave = False
        load()
        if not self._names and not cfg.write:
            k = Kernel().load(j(cfg.workdir, "runtime", "kernel"))
            logging.warn("# using %s" % pathname(k._container.path))
            self._names.update(k._names)
            c = Config().load(j(cfg.workdir, "runtime", "cfg"))
            logging.warn("# using %s" % pathname(c._container.path))
            if "workdirs" in c and c.workdirs:
                    cfg.workdirs = c.workdirs
            elif cfg.scan:
                homedir = os.path.expanduser("~")
                cfg.workdirs = list(locatedir(homedir, 'runtime'))
                dosave = True
        if cfg.write:
            cfg.workdirs = []
            dosave = True
        if cfg.yes or cfg.write or not self._names:
            self.walk("bot")
            self.sync(j(cfg.workdir, "runtime", "kernel"))
        for name in cfg.mods.split(","):
            if name:
                self.walk(name)          
        if cfg.all or cfg.init:
            for modname in self.modules("bot"):
                n = modname.split(".")[-1]
                if modname in cfg.ignore and n not in cfg.init:
                    continue
                if cfg.all or n in cfg.init.split(","):
                    try:
                        thr = self.init(modname)
                    except:
                        logging.error(get_exception())
                        continue
                    thrs.append(thr)
        for modname in cfg.needed:
            try:
                thr = self.init(modname)
            except:
                logging.error(get_exception())
                continue
            thrs.append(thr)
        if not self._cmnds:
            self._cmnds = sorted(set([cmnd for cmnd in self._names.keys()]))
        if dosave:
            cfg.sync(j(cfg.workdir, "runtime", "cfg"))
        set_completer(self._cmnds)
        self.start(**kwargs)
        if cfg.args:
            e = self.once(" ".join(cfg.args))
            e.wait()
            self.ready()
            return self
        for bot in fleet:
            bot._connected.wait()
        results = self.waiter(thrs)
        for bot in fleet:
            bot.prompt()
        if cfg.shell or cfg.init:
            for object in results:
                try:
                    object.wait()
                except KeyboardInterrupt:
                    break
                except:
                    logging.error(get_exception())
        self.ready()
        return self
               
    def callcb(self, event): 
        try:
            _cbs = self._cbs[event.origin]
        except KeyError:
            _cbs = []
        for cb in _cbs:
            cb(event)

    def cmnd(self, txt):
        """ execute a command based on txt. """
        from bot.space import fleet
        from bot.raw import RAW
        bot = RAW()
        fleet.add(bot)
        event = Event()
        event.cc = ""
        event.origin = "root@shell"
        event.channel = "#mad"
        event.server = "localhost"
        event.btype = bot.type
        event.txt = txt
        event.parse()
        return event

    def direct(self, name, package=None):
        """ import a module directly, not storing it in the cache. """
        return importlib.import_module(name, package)

    def dispatch(self, event):
        """ dispatch an event. """
        try:
            event.handle()
        except:
            logging.error(get_exception())
        self._time.last = time.time()

    def get_funcs(self, cmnd):
        """ search for a function registered by command. """
        from bot.space import alias
        oldcmnd = cmnd
        cmnd = alias.get(cmnd, cmnd)
        funcs = self._handlers.get(cmnd, [])
        if not funcs:
            funcs = self._handlers.get(oldcmnd, [])
            if not funcs:
                modnames = self._names.get(cmnd, [])
                for modname in modnames:
                    self.loading(modname)
                funcs = self._handlers.get(cmnd, [])
        return funcs

    def init(self, modname):
        """ initialize a module. """
        event = Event()
        n = modname.split(".")[-1]
        mod = self._table.get(modname, None)
        if not mod or type(mod) == str:
            mod = self.load_mod(modname)
        if "init" in dir(mod):
            logging.warn("# init %s" % modname)
            return self.launch(mod.init, event)

    def list(self, name):
        """ list all functions found in a module. """
        for modname in self.modules(name):
            mod = self.direct(modname)
            for key in dir(mod):
                if key in ["init", "shutdown"]:
                    continue
                object = getattr(mod, key, None)
                if object and type(object) == types.FunctionType:
                    if "event" in object.__code__.co_varnames:
                        yield key

    def load_mod(self, modname, force=True):
        """ load a module and if force is sset to True, put in in the cache, """
        mod = self.direct(modname)
        if mod and force:
            self._table[modname] = mod
        return mod

    def loading(self, modname, force=True):
        """ load a module. """
        logging.warn("# loading %s" % modname)
        if force:
            mod = self.load_mod(modname)
        else:
            mod = self.direct(modname)
        res = Object()
        for key in dir(mod):
            if key.startswith("_"):
                continue
            object = getattr(mod, key, None)
            if object and type(object) == types.FunctionType:
                if "event" in object.__code__.co_varnames:
                   if key.startswith("cb_"): 
                       k = key.split("cb_")[-1]
                       self._cbs.register(key, object)
                   else:
                       self._names.register(key, modname)
                       if key not in ["init", "shutdown"]:
                           self._handlers.register(key, object)
                   res[key] = object
        return res

    def modules(self, name=""):
        """ return a list of modules in the named packages or cfg.packages if no module name is provided. """
        from bot.space import cfg
        if not name:
            names = cfg.packages
        else:
            names = [name, ]
        for name in names:
            package = self.direct(name)
            for pkg in pkgutil.walk_packages(package.__path__, name + "."):
                yield pkg[1]

    def once(self, txt):
        """ run once command. """
        e = self.cmnd(txt)
        self.dispatch(e)
        return e

    def put(self, *args, **kwargs):
        if args:
            logging.debug("# put %s" % args[0])
        super().put(*args, **kwargs)

    def register(self, key, value, force=True):
        """ register handlers. """
        self._handlers.register(key, value, force)

    def reload(self, name, force=False, event=None):
        """ reload module. """
        e = event or template.get("event")
        try:
            self.loading(name, True)
        except (KeyError, ImportError) as ex:
            pass
        try:
            self._table[name].shutdown(e)
        except (KeyError, AttributeError):
            pass
        try:
            self._table[name] = importlib.reload(self._table[name])
        except KeyError:
            pass
        except:
            logging.error(get_exception())
        try:
            self._table[name].init(e)
        except (KeyError, AttributeError):
            pass
        if name in self._table:
            return self._table[name]

    @locked
    def shutdown(self, close=True):
        """ stop bot, services and plugins. """
        from bot.space import cfg, exceptions, fleet, kernel, runtime
        logging.warn("")
        event = Event()
        event.origin = "root@shell"
        event.server = "kernel"
        thrs = []
        for bot in fleet.bots:
            if "stop" in dir(bot):
                bot.stop()
            elif "exit" in dir(bot):
                bot.exit()
            bot.ready()
        for key, mod in self._table.items():
            try:
                mod.shutdown(event)
            except AttributeError:
                continue
            except:
                logging.error(get_exception(str(mod)))
        #kernel.sync(j(cfg.workdir, "runtime", "kernel"))
        #cfg.sync(j(cfg.workdir, "runtime", "cfg"))
        self.ready()
        for line in exceptions:
            print(line)

    def walk(self, name, init=False, force=False):
        """ return all modules in a package. """
        mod = []
        logging.warn("# walk %s" % name)
        for modname in sorted(list(self.modules(name))):
            self.loading(modname, force)
            if init:
                self.init(modname)
            mod.append(modname)
        return mod
