# bot/output.py
#
#

"""
    commands that output data.

"""

from bot.name import name, tname
from bot.object import Object
from bot.space import cfg, kernel, fleet
from bot.time import elapsed

import bot.space

format = "%-4s %-30s %-8s %-7s %-13s %-20s"

import bot
import os
import threading
import time

def announce(event):
    """ announce text on all channels in fleet. """
    kernel.announce(event._parsed.rest)

def cmnds(event):
    """ show list of commands. """
    res = sorted(set([cmnd for cmnd, mod in kernel._names.items() if event._parsed.rest in str(mod)]))
    event.reply(res or "no modules loaded.")

def ls(event):
    """ show subdirs in working directory. """
    event.reply(" ".join(os.listdir(cfg.workdir)))

def pid(event):
    """ show pid of the MADBOT bot. """
    event.reply(str(os.getpid()))

def ps(event):
    """ show running threads. """
    res = []
    nr = 1
    txt = []
    k = bot.space.kernel
    uptime = elapsed(int(time.time() - k._time.start))
    event.reply("KERNEL")
    event.reply(format % (0, "kernel", k._state.printable(nokeys=True), uptime, k._counter.printable(), k._error.printable(nokeys=True)))
    for thr in sorted(k.running(), key=lambda x: tname(x)):
        object = Object(vars(thr))
        if "start" not in object._time:
            continue
        if "sleep" in object:
            uptime = elapsed(object.sleep - int(time.time() - object._time.latest))
        else:
            uptime = elapsed(int(time.time() - object._time.start))
        thrname = tname(thr)
        res = format % (nr, thrname, object._state.printable(nokeys=True), uptime, object._counter.printable(), object._error.printable(nokeys=True))
        event.reply(res.rstrip())
        nr += 1
    event.reply("")
    event.reply("FLEET")
    for obj in fleet:
        uptime = elapsed(int(time.time() - obj._time.start))
        event.reply(format % (nr, obj.id(), obj._state.printable(nokeys=True), uptime,obj._counter.printable(), obj._error.printable(nokeys=True)))
        nr += 1

def show(event):
    import bot.space
    if not event._parsed.rest:
        event.reply("choose one of %s" % ",".join(bot.space.__all__))
        return
    try:
        item, value = event._parsed.rest.split(" ", 1)
    except:
        item = event._parsed.rest
        value = None
    if item == "bot":
        bot = fleet.get_bot(event.id())
        event.reply(bot)
        return
    if item == "fleet":
        for bot in fleet:
            event.reply(bot.id())
        return
    obj = getattr(bot.space, item, None)
    if value:
        val = getattr(obj, value, None)
        event.reply(val)
    else:
        event.reply(obj)

def test(event):
    """ echo origin. """
    from bot.misc import stripped
    event.reply("hello %s" % stripped(event.origin))

def uptime(event):
    """ show uptime. """
    event.reply("uptime is %s" % elapsed(time.time() - bot._starttime))

def version(event):
    """ show version. """
    from bot import __version__, __txt__
    event.reply("MADBOT #%s - %s" % (__version__, __txt__))

def whoami(event):
    """ show origin. """
    event.reply(event.origin)
