# mad/rss.py
#
#

""" rss module. """

from bot.compose import compose
from bot.join import sj, j
from bot.name import n
from bot.time import file_time, to_time
from bot.url import get_feed

from bot.clock import Repeater
from bot.db import Db
from bot.object import Default, Object
from bot.register import Register
from bot.launch import Launcher

from bot.space import kernel, runtime

import logging
import time
import bot

class Feed(Object):

    pass

class RSS(Register, Launcher):

    #path = j("runtime", "rss")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._type = str(type(self))

    def start(self, *args, **kwargs):
        from bot.space import runtime
        repeater = Repeater(600, self.fetcher)
        runtime.register("RSS", self)
        return self.launch(repeater.start)

    def stop(self):
        self.kill("RSS")
       
    def fetcher(self):
        from bot.space import kernel, seen
        thrs = []
        nr = len(seen.urls)
        for object in kernel.find("rss"):
            if "rss" not in object:
                continue
            if not object.rss:
                continue
            thr = kernel.launch(self.fetch, object)
            thrs.append(thr)
        result = kernel.waiter(thrs)
        if result:
            logging.info("! fetched %s" % (sj(*[x for x in result if x])))
            seen.sync()
        return result

    def synchronize(self):
        from bot.space import kernel, runtime, seen
        nr = 0
        for object in kernel.find("rss"):
            if not object.get("rss", None):
               continue
            for o in get_feed(object.rss):
                if o.link in seen.urls:
                    continue
                seen.urls.append(o.link)
                nr += 1
        logging.warn("# %s feeds" % nr)
        seen.sync()
        return seen

    def fetch(self, object):
        from bot.space import fleet, kernel, seen
        nr = 0
        self._time.polls += 1
        for o in get_feed(object.rss):
            if o.link in seen.urls:
                continue
            seen.urls.append(o.link)
            feed = Feed(o)
            feed.services = "rss"
            for f in self.cfg.save_list:
                if f in feed.link:
                    if "published" in feed:
                        try:
                            date = file_time(to_time(feed.published))
                            feed.save(stime=date)
                        except bot.error.ENODATE as ex:
                            logging.warn("ENODATE %s" % str(ex))
                    else:
                        feed.save()
            for bot in fleet:
                bot.announce(self.display(feed))
            nr += 1
        return nr 

    def display(self, object):
        result = ""
        for check in self.cfg.descriptions:
            if check in object.link:
                summary = object.get("summary", None)
                if summary:
                    result += "%s - " % summary
        for key in self.cfg.display_list:
            data = object.get(key, None)
            if data:
                result += "%s - " % data.rstrip()
        if result:
            return result[:-3].rstrip()

def init(event):
    rss = RSS()
    rss.start()
    return rss

def shutdown(event):
    rss = runtime.get("RSS", [])
    for item in rss:
        item.stop()

def synchronize(event):
    clients = runtime.get("RSS", [])
    for client in clients:
        c = compose(client)
        seen = c.synchronize()
        event.reply("%s urls updated" % len(seen.urls))

def fetcher(event):
    clients = runtime.get("RSS", [])
    for client in clients:
        c = compose(client)
        res = c.fetcher()
        event.reply("fetched %s feeds" % ",".join(res))
