# mad/shell.py
#
#

""" function concerning the command line options of the bot. """

from bot.log import YELLOW, GREEN, BOLD, ENDC
from bot.object import Config, Object

from bot.template import template

import bot

import rlcompleter
import threading
import optparse
import readline
import termios
import logging
import atexit
import sys
import os

histfile = os.path.expanduser("~/.history")

resume = Object()

def banner(descr, version, txt, colors=False, *args, **kwargs):
    try:
        if colors:
            print("%s%s%s #%s%s - %s%s%s\n" % (BOLD, YELLOW, descr, version, ENDC, GREEN, txt, ENDC))
        else:
            print("%s #%s - %s\n" % (descr, version, txt))
    except Exception as ex:
        print(ex)
        print('try setting PYTHONIOENCODING="utf-8" before starting this program.')

def parse_cli(*args, **kwargs):
    opts, arguments = make_opts(opts_defs)
    cfg = Config()
    cfg.template("kernel")
    cfg.args = arguments
    cfg.update(vars(opts))
    return cfg

def getline(prompt="> "):
    import termios, sys
    fd = sys.stdin.fileno()
    old = termios.tcgetattr(fd)
    new = termios.tcgetattr(fd)
    try:
        termios.tcsetattr(fd, termios.TCSADRAIN, new)
        line = input(prompt)
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old)
    return line

def termsetup(fd):
    old = termios.tcgetattr(fd)
    return old

def termreset(fd, old):
    termios.tcsetattr(fd, termios.TCSADRAIN, old)

class Completer(rlcompleter.Completer):

    def __init__(self, options):
        super().__init__()
        self.options = options
 
    def complete(self, text, state):
        if state == 0:
            if text:
                self.matches = [s for s in self.options if s and s.startswith(text)]
            else:
                self.matches = self.options[:]
        try:
            return self.matches[state]
        except IndexError:
            return None

def set_completer(optionlist):
    import readline
    readline.parse_and_bind("tab: complete")
    completer = Completer(optionlist)
    readline.set_completer(completer.complete)
    atexit.register(lambda: readline.set_completer(None))

def enable_history():
    if not os.path.exists(histfile):
        bot.file.touch(histfile)
    readline.read_history_file(histfile)
    atexit.register(close_history)

def close_history():
    readline.write_history_file(histfile)

def reset():
    close_history()
    if resume.old:
        termreset(resume.fd, resume.old)
    
def startup():
    global resume
    try:
        resume.fd = sys.stdin.fileno()
        resume.old = termsetup(sys.stdin.fileno())
    except:
        resume.fd = None
        resume.old = None
    atexit.register(reset)

def make_opts(options):
    parser = optparse.OptionParser(usage='usage: %prog [options]', version=str(bot.__version__))
    for option in options:
        type, default, dest, help = option[2:]
        if "store" in type:
            try: parser.add_option(option[0], option[1], action=type, default=default, dest=dest, help=help)
            except Exception as ex: logging.error("^ Opts/error %s option %s" % (str(ex), option)) ; continue
        else:
            try: parser.add_option(option[0], option[1], type=type, default=default, dest=dest, help=help)
            except Exception as ex: logging.error("^ Opts/error %s option %s" % (str(ex), option)) ; continue
    args = parser.parse_args()
    return args

def list_eggs(filter="bot"):
    for f in sys.path:
        if ".egg" not in f:
            continue
        if filter and filter not in f:
            continue
        yield f

def list_path(filter=""):
    for f in sys.path:
        if filter and filter not in f:
            continue
        yield f

def show_eggs(filter="mad"):
    path = list(list_eggs(filter))[0]
    return path

def reboot():
    os.execl(sys.argv[0], *(sys.argv + ["-r"]))

opts_defs = [
              ('-a', '--all', 'store_true', False, 'all', "load all plugins."),
              ('-b', '--banner', 'store_true', False, 'banner', "show banner"),
              ('-c', '--colors', 'store_true', False, 'colors', "turn on color mode."),
              ('-d', '--workdir', 'string',  "", 'workdir', "working directory."),
              ('-e', '--onerror', 'store_true', False, 'onerror', "raise on error."),
              ('-g', '--global', 'store_false', True, 'local', "use ~/.madsbot as workdir"),
              ('-i', '--init', 'string', "", 'init', "whether to initialize plugins."),
              ('-l', '--loglevel', 'string', "", 'loglevel', "loglevel."),
              ('-m', '--mods', 'string', "", 'mods', "modules to load."),
              ('-o', '--owner', 'string', "root@shell", 'owner', "userhost/JID of the bot owner."),
              ('-p', '--port', 'string', 10102, 'port', "port to run HTTP server on."),
              ('-r', '--resume', 'store_true', False, 'resume', "resume on restart."),
              ('-s', '--shell', 'store_true', False, 'shell', "start a shell."),
              ('-t', '--test', 'store_true', False, 'test', "switch to test mode"),
              ('-v', '--verbose', 'store_true', False, 'verbose', 'use verbose mode.'),
              ('-w', '--write', 'store_true', False, 'write', 'save kernel state after boot.'),
              ('-x', '--exclude', 'string', "test", 'exclude', "modules to exclude."),
              ('-y', '--yes', 'store_true', False, 'yes', "enable all boot options."),
              ('-z', '--reboot', 'store_true', False, 'reboot', "enable rebooting."),
              ('', '--homedir', 'string', os.path.expanduser("~"), 'homedir', "user's homedir"),
              ('', '--daemon', 'store_true', False, 'daemon', "switch to daemon mode."),
              ('', '--debug', 'store_true', False, 'debug', "enable debug mode."),
              ('', '--license', 'store_true', False, 'license', "show license."),
              ('', '--scan', 'store_true', False, 'scan', "scan for workdirs in the homedir."),
          ]

opts_defs_sed = [
              ('-d', '--dir', 'string', "", 'dir_sed', "directory to work with."),
              ('-l', '--loglevel', 'string', "error", 'loglevel', "loglevel"),
          ]  

opts_defs_udp = [
              ('-p', '--port', 'string', "10102", 'port', "port to run API server on"),
              ('-l', '--loglevel', 'string', "error", 'loglevel', "loglevel"),
          ]

opts_defs_doctest = [
              ('-e', '--onerror', 'store_true', False, 'onerror', "raise on error"),
              ('-v', '--verbose', 'store_true', False, 'verbose', "use verbose"),
              ('-l', '--loglevel', 'string', "error", 'loglevel', "loglevel"),
          ]

startup()
