# mod/system.py
#
#

""" system related commands. """

from bot.log import loglevel as _loglevel
from bot.shell import reboot as boot
from bot.shell import reset

from bot.space import alias, cfg, kernel, runtime, users

import _thread

def loglevel(event):
    """ set loglevel. """
    level = event._parsed.rest
    if not level:
        event.reply(cfg.loglevel)
        return
    cfg.loglevel = level
    kernel.sync()
    _loglevel(level, cfg.colors)

def exit(event):
    """ stop the bot. """
    if not users.allowed(event.origin, "OPER"):
        event.reply("you are not allowed to give the exit command. ")
        return
    _thread.interrupt_main()

def reload(event):
    """ reload a plugin. """
    if not users.allowed(event.origin, "OPER"):
        event.reply("you are not allowed to give the reload command.")
        return
    if not event._parsed.rest:
        event.reply(",".join([x.split(".")[-1] for x in kernel.modules()]))
        return
    user = kernel.last("user", event.origin) 
    for modname in kernel.modules():
        if event._parsed.rest not in modname:
            continue
        try:
            event.reply(kernel.reload(modname, True))
        except ImportError as ex:
            event.reply(str(ex))

def start(event):
    """ start a plugin. """
    if not users.allowed(event.origin, "OPER"):
        event.reply("you are not allowed to give the start command. ")
        return
    if not event._parsed.rest:
        res = set([x.split(".")[-1] for x in kernel._table.keys()])
        event.reply(sorted(res))
        return
    name = "bot.%s" % event._parsed.args[0]
    mod = kernel.reload(name, force=True, event=event)
    if mod:
        event.ok(sname(mod).lower())
    else:
        event.reply("not ok")

def stop(event):
    """ stop a plugin. """
    if not users.allowed(event.origin, "OPER"):
        event.reply("you are not allowed to give the stop command.")
        return
    if not event._parsed.rest:
        event.reply("stop what ?")
        return
    name = "bot.%s" % event._parsed.args[0]
    try:
        mod = kernel._table[name]
        if "stop" in dir(mod):
            mod.stop(event)
        if "exit" in dir(mod):
            mod.exit(event)
        if "shutdown" in dir(mod):
            mod.shutdown(event)
    except (KeyError, ImportError):
        event.reply("no %s module found." % name)
        return
    kernel.kill(name.upper())
    event.reply("stopped %s" % name)

def save(event):
    """ make a kernel dump. """
    from bot.space import alias, kernel, seen
    alias.sync()
    kernel.sync()
    seen.sync()
    event.reply("saved")

def real_reboot(x):
    """ actual reboot. """
    kernel.shutdown()
    reset()
    boot()

def reboot(event):
    """ reboot the bot, allowing statefull reboot (keeping connections alive). """
    if not cfg.reboot:
        event.reply("# reboot is not enabled.")
        return
    if not users.allowed(event.origin, "OPER"):
        event.reply("you are not allowed to give the reboot command. ")
        return
    event.announce("rebooting")
    real_reboot("bla")
