# bot/time.py
#
#

""" plugin with time related functions. """

import bot.error

import datetime
import random
import time
import re
import os

def fn_time(daystr):
    """ determine the time used in a MADBOT filename. """
    daystr = daystr.replace("_", ":")
    datestr = " ".join(daystr.split(os.sep)[-2:])
    datestr = datestr.split(".")[0]
    try:
        t = time.mktime(time.strptime(datestr, "%Y-%m-%d %H:%M:%S"))
    except ValueError:
        t = 0
    return t

def now():
    """ turn a datetime string of the current time. """
    return str(datetime.datetime.now())

def rtime():
    """ return a filestamp usable in a filename. """
    return str(datetime.datetime.now()).replace(" ", os.sep)

def hms():
    """ return hour:minutes:seconds of toda=y. """
    return str(datetime.datetime.today()).split()[1].split(".")[0]

def day():
    """" return the current day. """
    return str(datetime.datetime.today()).split()[0]

def year():
    """ return the year we are living in. """
    return str(datetime.datetime.now().year)

def get_time(txt):
    """ get time from a string containing day and/or hour. """
    try:
        target = get_day(txt)
    except bot.error.ENODATE:
        target = to_day(day())
    hour =  get_hour(txt)
    if hour:
        target += hour
    return target

def parse_time(txt):
    """" parse a string for a time mentioned. also parse for a diff in seconds. """
    seconds = 0
    target = 0
    txt = str(txt)
    for word in txt.split():
        if word.startswith("+"):
            seconds = int(word[1:])
            return time.time() + seconds
        if word.startswith("-"):
            seconds = int(word[1:])
            return time.time() - seconds
    if not target:
        try:
            target = get_day(txt)
        except bot.error.ENODATE:
            target = to_day(day())
        hour =  get_hour(txt)
        if hour:
            target += hour
    return target

def extract_time(daystr):
    """ use standard time timeformats to extract a time from a string. """
    for format in year_formats:
        try:
            res = time.mktime(time.strptime(daystr, format))
        except:
            res = None
        if res:
            return res

def to_day(daystr):
    """ try to detect a time in a string. """
    previous = ""
    line = ""
    daystr = str(daystr)
    for word in daystr.split():
        line = previous + " " + word
        previous = word
        try:
            res = extract_time(line.strip())
        except ValueError:
            res = None
        if res:
            return res
        line = ""

def to_time(daystr):
    """
         convert time/date string to a unix timestamp

         example: 2016-08-29 16:34:23.837288
         example: Sat Jan 14 00:02:29 2017

    """
    daystr = str(daystr)
    daystr = daystr.split(".")[0]
    daystr = daystr.replace("GMT", "")
    daystr = daystr.replace("_", ":")
    daystr = " ".join([x.capitalize() for x in daystr.split() if not x[0] in ["+", "-"]])
    res = 0
    try:
        res = time.mktime(time.strptime(daystr, "%a, %d %b %Y %H:%M:%S"))
    except:
        pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%a, %d %b %Y %H:%M:%S %z"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%a, %d %b %Y %H:%M:%S %z"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%a %d %b %H:%M:%S %Y"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%a %d %b %H:%M:%S %Y %z"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%Y-%m-%d %H:%M:%S"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%d-%m-%Y %H:%M:%S"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%d-%m-%Y %H:%M"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%Y-%m-%d %H:%M"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%Y-%m-%d"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%d-%m-%Y"))
        except:
            pass
    if not res:
        try:
            res = time.mktime(time.strptime(daystr, "%d %m %Y"))
        except:
            pass
    if not res:
        raise bot.error.ENODATE(daystr)
    return res

def file_time(timestamp):
    """ return a pseudo random time on today's start of the day. """
    return str(datetime.datetime.fromtimestamp(timestamp)).replace(" ", os.sep) + "." + str(random.randint(111111,999999))

def to_date(*args, **kwargs):
    """ convert to date. """
    date = args[0]
    if not date:
        return None
    date = date.replace("_", ":")
    res = date.split()
    ddd = ""
    try:
        if "+" in res[3]: 
            raise ValueError
        if "-" in res[3]:
            raise ValueError
        int(res[3])
        ddd = "{:4}-{:#02}-{:#02} {:6}".format(res[3], monthint[res[2]], int(res[1]), res[4])
    except (IndexError, KeyError, ValueError):
        try:
            if "+" in res[4]:
                raise ValueError
            if "-" in res[4]:
                raise ValueError
            int(res[4])
            ddd = "{:4}-{:#02}-{:02} {:6}".format(res[4], monthint[res[1]], int(res[2]), res[3])
        except (IndexError, KeyError, ValueError):
            try:
                ddd = "{:4}-{:#02}-{:02} {:6}".format(res[2], monthint[res[1]], int(res[0]), res[3])
            except (IndexError, KeyError):
                try:
                    ddd = "{:4}-{:#02}-{:02}".format(res[2], monthint[res[1]], int(res[0]))
                except (IndexError, KeyError):
                    try:
                        ddd = "{:4}-{:#02}".format(res[2], monthint[res[1]])
                    except (IndexError, KeyError):
                        try:
                            ddd = "{:4}".format(res[2])
                        except (IndexError, KeyError):
                            ddd = ""
    return ddd

def elapsed(seconds, short=True):
    """ return a string showing the elapsed days, hours, minutes, seconds. """
    txt = ""
    sub = str(seconds).split(".")[-1]
    nsec = float(seconds)
    year = 365*24*60*60
    week = 7*24*60*60
    day = 24*60*60
    hour = 60*60
    minute = 60
    years = int(nsec/year)
    nsec -= years*year
    weeks = int(nsec/week)
    nsec -= weeks*week
    days = int(nsec/day)
    nsec -= days*day
    hours = int(nsec/hour)
    nsec -= hours*hour
    minutes = int(nsec/minute)
    sec = nsec - minutes*minute
    if years:
        txt += "%sy" % years
    if weeks:
        days += weeks * 7
    if days:
        txt += "%sd" % days
    if years and short and txt:
        return txt
    if hours:
        txt += "%sh" % hours 
    if days and short and txt:
        return txt
    if minutes:
        txt += "%sm" % minutes
    if hours and short and txt:
        return txt
    if sec == 0:
        txt += "0s"
    elif sec < 1 or not short:
        txt += "%.3fs" % sec
    else:
        txt += "%ss" % int(sec)
    txt = txt.strip()
    return txt

def today():
    """" return the day of a MADBOT filename. """
    t = rtime().split(".")[0]
    ttime = time.strptime(t, "%Y-%m-%d/%H:%M:%S")
    result = time.mktime(ttime)
    return result

def get_day(daystr):
    """ get the day from the string provided. """
    day = 0
    try:
        ymdre = re.search('(\d+)-(\d+)-(\d+)', daystr)
        if ymdre:
            (year, month, day) = ymdre.groups()
    except:
        try:
            ymre = re.search('(\d+)-(\d+)', daystr)
            if ymre:
                (year, month) = ymre.groups()
                day = 1
        except:
            raise bot.error.ENODATE(daystr)
    if not day:
        raise bot.error.ENODATE(daystr)
    day = int(day)
    month = int(month)
    year = int(year)
    date = "%s %s %s" % (day, bdmonths[month], year)
    return time.mktime(time.strptime(date, "%d %b %Y"))

def get_hour(daystr):
    """ get the hour from the string provided. """
    try:
        hmsre = re.search('(\d+):(\d+):(\d+)', str(daystr))
        hours = 60 * 60 * (int(hmsre.group(1)))
        hoursmin = hours  + int(hmsre.group(2)) * 60
        hms = hoursmin + int(hmsre.group(3))
    except AttributeError:
        pass
    except ValueError:
        pass
    try:
        hmre = re.search('(\d+):(\d+)', str(daystr))
        hours = 60 * 60 * (int(hmre.group(1)))
        hms = hours + int(hmre.group(2)) * 60
    except AttributeError:
        return 0
    except ValueError:
        return 0
    return hms

## FORMATS

year_formats = [
    "%Y-%m-%d",
    "%d-%m-%Y",
    "%d-%m",
    "%m-%d",
]
timere = re.compile('(\S+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+)')
bdmonths = ['Bo', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
monthint = {
    'Jan': 1,
    'Feb': 2,
    'Mar': 3,
    'Apr': 4,
    'May': 5,
    'Jun': 6,
    'Jul': 7,
    'Aug': 8,
    'Sep': 9,
    'Oct': 10,
    'Nov': 11,
    'Dec': 12 
}
